@@ -27,22 +27,23 @@ from gwaslab.filtervalue import filterin
 from gwaslab.filtervalue import filterregionin
 from gwaslab.filtervalue import filterregionout
 from gwaslab.filtervalue import inferbuild
+from gwaslab.filtervalue import sampling
 from gwaslab.mqqplot import mqqplot
 from gwaslab.calculate_gc import gc
 from gwaslab.getsig import getsig
+from gwaslab.getsig import annogene
+from gwaslab.getsig import getnovel
 from gwaslab.fill import filldata
-from gwaslab.to_formats import tobed
-from gwaslab.to_formats import tofuma
-from gwaslab.to_formats import toldsc
-from gwaslab.to_formats import tossf
+from gwaslab.to_formats import tofmt
 from gwaslab.get_hapmap3 import gethapmap3
 from gwaslab.compare_effect import plotdaf
-from gwaslab.CommonData import get_chr_NC_dict
 from gwaslab.CommonData import get_chr_list
 from gwaslab.CommonData import get_number_to_chr
 from gwaslab.CommonData import get_chr_to_number
 from gwaslab.CommonData import get_high_ld
 from gwaslab.CommonData import get_format_dict
+from gwaslab.CommonData import get_formats_list
+import gc
 
 #20220309
 class Sumstats():
@@ -55,6 +56,8 @@ class Sumstats():
              pos=None,
              ea=None,
              nea=None,
+             ref=None,
+             alt=None,
              eaf=None,
              neaf=None,
              n=None,
@@ -66,7 +69,6 @@ class Sumstats():
              mlog10p=None,
              info=None,
              OR=None,
-             OR_se=None,
              OR_95L=None,
              OR_95U=None,
              status=None,
@@ -77,7 +79,8 @@ class Sumstats():
              **readargs):
         
         self.build = build
-        self.meta = {"Genome build":build}
+        self.meta = {"Genome build":build,
+                 "Name":"Sumstats_1","StudyType":"Unknown"}
         self.data = pd.DataFrame()
         self.log = Log()
         
@@ -91,6 +94,8 @@ class Sumstats():
           pos=pos,
           ea=ea,
           nea=nea,
+          ref=ref,
+          alt=alt,
           eaf=eaf,
           neaf=neaf,
           n=n,
@@ -102,7 +107,6 @@ class Sumstats():
           mlog10p=mlog10p,
           info=info,
           OR=OR,
-          OR_se=OR_se,
           OR_95L=OR_95L,
           OR_95U=OR_95U,
           direction=direction,
@@ -112,7 +116,7 @@ class Sumstats():
           verbose=verbose,
           readargs=readargs,
           log=self.log)
-                   
+        gc.collect()     
 #### healper #################################################################################
     #value to arguement name 
     def get_columns(self, columns=[]):
@@ -133,7 +137,6 @@ class Sumstats():
             "MLOG10P": "mlog10p",
             "INFO": "info",
             "OR": "OR",
-            "OR_SE": "OR_se",
             "OR_95L": "OR_95L",
             "OR_95U": "OR_95U",
         }
@@ -233,7 +236,8 @@ class Sumstats():
             self.data = parallelnormalizeallele(self.data,log=self.log,n_cores=n_cores,**normalizeallele_args)
             
             self.data = sortcolumn(self.data,log=self.log)
-        
+            
+            gc.collect()
         
         #####################################################
         #part 2 : annotating and flipping
@@ -254,22 +258,28 @@ class Sumstats():
             
             self.data = flipallelestats(self.data,log=self.log,**flipallelestats_args)
             
+            gc.collect()
+            
         if ref_infer is not None: 
             
             self.data= parallelinferstrand(self.data,ref_infer = ref_infer,ref_alt_freq=ref_alt_freq,
                                               n_cores=n_cores,log=self.log,**inferstrand_args)
             
             self.data =flipallelestats(self.data,log=self.log,**flipallelestats_args)
+            
+            gc.collect()
         
         #####################################################
         if ref_rsid_tsv is not None:
             
             self.data = parallelizeassignrsid(self.data,path=ref_rsid_tsv,ref_mode="tsv",
                                                  n_cores=n_cores,log=self.log,**assignrsid_args)
+            gc.collect()
         if ref_rsid_vcf is not None:
             
             self.data = parallelizeassignrsid(self.data,path=ref_rsid_vcf,ref_mode="vcf",
-                                                 n_cores=n_cores,log=self.log,**assignrsid_args)     
+                                                 n_cores=n_cores,log=self.log,**assignrsid_args)   
+            gc.collect()
         ######################################################    
         if remove is True:
             
@@ -279,7 +289,7 @@ class Sumstats():
         self.data = sortcoordinate(self.data,log=self.log)
         
         self.data = sortcolumn(self.data,log=self.log)
-        
+        gc.collect()
         return self
     ############################################################################################################
     #customizable API to build your own QC pipeline
@@ -351,7 +361,13 @@ class Sumstats():
         if inplace is False:
             return filterregionout(self.data,**args)
         else:
-            self.data = filterregionout(self.data,log=self.log,**args)        
+            self.data = filterregionout(self.data,log=self.log,**args)
+    
+    def random_variants(self,in_place=False,n=1,**args):
+        if in_place is True:
+            self.data = sampling(self.data,n=n,log=self.log,**args)
+        else:
+            return sampling(self.data,n=n,log=self.log,**args)
     
 ###########################################################################################################################            
     def check_af(self,**args):
@@ -361,7 +377,23 @@ class Sumstats():
         plot = plotdaf(self.data, **args)
         
     def plot_mqq(self, **args):
-        plot = mqqplot(self.data, chrom="CHR", pos="POS", p="P", **args)
+
+        chrom="CHR"
+        pos="POS"
+        p="P"
+        
+        if "SNPID" in self.data.columns:
+            snpid="SNPID"
+        elif "rsID" in self.data.columns:
+            snpid="rsID"
+        
+        if "EAF" in self.data.columns:
+            eaf="EAF"
+        else:
+            eaf=None
+        
+        plot = mqqplot(self.data,snpid=snpid, chrom=chrom, pos=pos, p=p, eaf=eaf,**args)
+        
         return plot
 
     def get_lead(self, **args):
@@ -378,8 +410,34 @@ class Sumstats():
                            **args)
         # return sumstats object    
         return output
+    def get_novel(self, **args):
+        if "SNPID" in self.data.columns:
+            id_to_use = "SNPID"
+        else:
+            id_to_use = "rsID"
+        output = getnovel(self.data,
+                           id=id_to_use,
+                           chrom="CHR",
+                           pos="POS",
+                           p="P",
+                           log=self.log,
+                           **args)
+        # return sumstats object    
+        return output
 
-    
+    def anno_gene(self, **args):
+        if "SNPID" in self.data.columns:
+            id_to_use = "SNPID"
+        else:
+            id_to_use = "rsID"
+        output = annogene(self.data,
+                           id=id_to_use,
+                           chrom="CHR",
+                           pos="POS",
+                           log=self.log,
+                           **args)
+        return output
+        
     def get_per_snp_h2(self,**args):
         self.data = getpersnph2(self.data,beta="BETA",af="EAF",**args)
         #add data inplace
@@ -429,17 +487,23 @@ class Sumstats():
               hapmap3=False,
               exclude_hla=False,  
               build="19", 
+              n=None,
               verbose=True,
               output_log=True,
               to_csvargs={},
-              float_formats={}):
+              float_formats={},
+              xymt_number=False,
+              xymt=["X","Y","MT"],
+              chr_prefix=None,
+              bgzip=False,
+              tabix=False):
         
-        if fmt in ["ldsc","fuma","metal","bed","vcf","fastgwa","ssf","plink","plink2","saige","regenie","gwascatalog","pgscatalog"]:
-            if verbose: self.log.write("Output sumstats to format:",fmt)
+        if fmt in get_formats_list() + ["vep","bed","annovar","vcf"]:
+            if verbose: self.log.write("Start to format the output sumstats in: ",fmt, " format")
         else:
             raise ValueError("Please select a format to output")
         
-        suffix=fmt
+        
         #######################################################################################################
         # filter
         output = self.data.copy()
@@ -448,7 +512,11 @@ class Sumstats():
 
         if exclude is not None:
             output = output.loc[~output[id_use].isin(exclude),:]
-
+        
+        #hla and hapmap3 #######################################################################################
+        suffix=fmt
+        
+        #exclude hla
         if exclude_hla is True:
             if verbose: self.log.write(" -Excluding variants in HLA region ...")
             before = len(output)
@@ -458,28 +526,35 @@ class Sumstats():
             if verbose: self.log.write(" -Exclude "+ str(before - after) + " variants in HLA region.")
             suffix = "noMHC."+suffix
         
+        #extract hapmap3 SNPs
         if hapmap3 is True:
             output = gethapmap3(output,build=build,verbose=True)
             after = len(output)
             if verbose: self.log.write(" -Extract "+ str(after) + " variants in Hapmap3 datasets for build "+build+".")
             suffix = "hapmap3."+suffix
+        
+        # add a n column
+        if n is not None:
+            output["N"] = n
                 
         #######################################################################################################
         #formatting statistics
         if verbose: self.log.write(" -Formatting statistics ...")
+        
         formats = {'EAF': '{:.4g}', 
-                   'BETA': '{:.4f}', 
-                   'Z': '{:.4f}',
-                   'CHISQ': '{:.4f}',
-                   'SE': '{:.4f}',
-                   'OR': '{:.4f}',
-                   'OR_95U': '{:.4f}',
-                   'OR_95L': '{:.4f}',
-                   'INFO': '{:.4f}',
-                   'P': '{:.4e}',
-                   'MLOG10P': '{:.4f}',
-                   'DAF': '{:.4f}'
+                'BETA': '{:.4f}', 
+                'Z': '{:.4f}',
+                'CHISQ': '{:.4f}',
+                'SE': '{:.4f}',
+                'OR': '{:.4f}',
+                'OR_95U': '{:.4f}',
+                'OR_95L': '{:.4f}',
+                'INFO': '{:.4f}',
+                'P': '{:.4e}',
+                'MLOG10P': '{:.4f}',
+                'DAF': '{:.4f}'
                   }
+        
         for col, f in float_formats.items():
             if col in output.columns: 
                 formats[col]=f
@@ -488,7 +563,7 @@ class Sumstats():
                 if output[col].dtype in ["float64","float32","float16","float"]:
                     output[col] = output[col].map(f.format)
         if verbose: 
-            self.log.write(" -Float statistics formats:")  
+            self.log.write(" - Float statistics formats:")  
             keys=[]
             values=[]
             for key,value in formats.items():
@@ -497,18 +572,25 @@ class Sumstats():
                     values.append(value)
             self.log.write("  - Columns:",keys) 
             self.log.write("  - Output formats:",values) 
+            
         ##########################################################################################################          
         # output, mapping column names
-        if fmt=="ldsc":
-            toldsc(output, path=path+"."+suffix,verbose=True,log=self.log,to_csvargs=to_csvargs)
-        #if fmt=="fuma":
-            #toldsc(output, path=path+"."+fmt,verbose=True,log=self.log,to_csvargs=to_csvargs)
-        #if fmt=="bed":
-            #toldsc(output, path=path+"."+fmt,verbose=True,log=self.log,to_csvargs=to_csvargs)
-        if fmt=="vcf":
-            toldsc(output, path=path+"."+suffix,verbose=True,log=self.log,to_csvargs=to_csvargs)
-        if fmt in ["fastgwa","ssf","plink","plink2","saige","regenie","gwascatalog","pgscatalog"]:
-            to_formats.tofmt(output, path=path,fmt=fmt,suffix=suffix,verbose=True,log=self.log,to_csvargs=to_csvargs)
+        
+        if fmt in get_formats_list() + ["vep","bed","annovar","vcf"]:
+            tofmt(output,
+                  path=path,
+                  fmt=fmt,
+                  suffix=suffix,
+                  build=build,
+                  verbose=True,
+                  log=self.log,
+                  to_csvargs=to_csvargs,
+                  chr_prefix=chr_prefix,
+                  meta = self.meta,
+                  bgzip=bgzip,
+                  tabix=tabix,
+                  xymt_number=xymt_number,
+                  xymt=xymt)
         if output_log is True:
-            if verbose: self.log.write("Saving log file...")
-            self.log.save(path + "."+ suffix +".log")
+            if verbose: self.log.write(" -Saving log file:",path + "."+ suffix +".log")
+        if verbose: self.log.write("Finished outputting successfully!")
