@@ -177,33 +177,48 @@ def filterregionout(sumstats, path=None, chrom="CHR",pos="POS", high_ld=False, b
     return sumstats
 
 def inferbuild(sumstats,status="STATUS",chrom="CHR", pos="POS", ea="EA", nea="NEA",build="19", verbose=True,log=Log()):
-    if verbose:log.write(" -Start to infer genome build version using hapmap3 SNPs...")    
-    data_path_19 =  path.dirname(__file__) + '/data/hapmap3_rs_chr_pos_a1_a2/hapmap3_db150_hg19.snplist.gz'    
-    data_path_38 =  path.dirname(__file__) + '/data/hapmap3_rs_chr_pos_a1_a2/hapmap3_db151_hg38.snplist.gz'    
+    if verbose:log.write("Start to infer genome build version using hapmap3 SNPs...")    
+    data_path_19 =  path.dirname(__file__) + '/data/hapmap3_SNPs/hapmap3_db150_hg19.snplist.gz'    
+    data_path_38 =  path.dirname(__file__) + '/data/hapmap3_SNPs/hapmap3_db151_hg38.snplist.gz'    
+    
     if verbose:log.write(" -Loading Hapmap3 variants data...")        
     hapmap3_ref_19 = pd.read_csv(data_path_19,sep="\s+",usecols=["#CHROM","POS"],dtype={"#CHROM":"string","POS":"string"})
     hapmap3_ref_38 = pd.read_csv(data_path_38,sep="\s+",usecols=["#CHROM","POS"],dtype={"#CHROM":"string","POS":"string"})
     
     if chrom in sumstats.columns and pos in sumstats.columns:
-        if verbose: log.write(" -chr:pos will be used for matching...")
+        if verbose: log.write(" -CHR:POS will be used for matching...")
         raw_chrpos = sumstats[chrom].astype("string")+":"+sumstats[pos].astype("string")
+        
         hapmap3_ref_19["chr:pos"] = hapmap3_ref_19["#CHROM"]+":"+hapmap3_ref_19["POS"]
         hapmap3_ref_38["chr:pos"] = hapmap3_ref_38["#CHROM"]+":"+hapmap3_ref_38["POS"]
+        
         match_count_for_19 = sum(raw_chrpos.isin(hapmap3_ref_19["chr:pos"].values))
         match_count_for_38 = sum(raw_chrpos.isin(hapmap3_ref_38["chr:pos"].values))
-        if verbose:log.write(" -Matching variants for hg19: num_hg19=",match_count_for_19)        
-        if verbose:log.write(" -Matching variants for hg38: num_hg38=",match_count_for_38) 
+        
+        if verbose:log.write(" -Matching variants for hg19: num_hg19 = ",match_count_for_19)        
+        if verbose:log.write(" -Matching variants for hg38: num_hg38 = ",match_count_for_38) 
+        
+        if max(match_count_for_19, match_count_for_38)<10000:
+            if verbose:log.write(" -Warning: please be cautious due to the limited number of variants.") 
+        
         if match_count_for_19 > match_count_for_38:
-            if verbose:log.write(" -Since num_hg19>num_hg38, assigning genome build hg19...") 
+            if verbose:log.write(" -Since num_hg19 >> num_hg38, assigning genome build hg19...") 
             sumstats.loc[:,status] = vchange_status(sumstats.loc[:,status],1,"9","1")
             sumstats.loc[:,status] = vchange_status(sumstats.loc[:,status],2,"9","9")
-        elif match_count_for_38<match_count_for_38:
-            if verbose:log.write(" -Since num_hg19>num_hg38, assigning genome build hg19...") 
+        elif match_count_for_19 < match_count_for_38:
+            if verbose:log.write(" -Since num_hg19 << num_hg38, assigning genome build hg38...") 
             sumstats.loc[:,status] = vchange_status(sumstats.loc[:,status],1,"9","3")
             sumstats.loc[:,status] = vchange_status(sumstats.loc[:,status],2,"9","8")
         else:
-            if verbose:log.write(" -Since num_hg19=num_hg38, unable to infer...") 
+            if verbose:log.write(" -Since num_hg19 = num_hg38, unable to infer...") 
         return sumstats
     else:
-        raise ValueError("Not enough information to match SNPs. Please check your sumstats...")
+        raise ValueError("Not enough information to match SNPs. Please check if CHR and POS columns are in your sumstats...")
+
+def sampling(sumstats,n,verbose=True,log=Log()):
+    if verbose:log.write("Start to randomly select variants from the sumstats...") 
+    if verbose:log.write(" -Number of variants selected from the sumstats:",n)
+    sampled = sumstats.sample(n=n)
+    if verbose:log.write("Finished sampling...")
+    return sampled
     
\ No newline at end of file
