@@ -32,7 +32,7 @@ from gwaslab.CommonData import get_chr_list
 
 def fixID(sumstats,
        snpid="SNPID",rsid="rsID",chrom="CHR",pos="POS",nea="NEA",ea="EA",status="STATUS",
-       fixchrpos=False,fixid=False,fixeanea=False,fixeanea_flip=False,fixsep=False,overwrite=False,verbose=True,log=Log()):  
+       fixchrpos=False,fixid=False,fixeanea=False,fixeanea_flip=False,fixsep=False,overwrite=False,verbose=True,forcefixid=False,log=Log()):  
     
     '''
     1. fx SNPid
@@ -152,8 +152,8 @@ def fixID(sumstats,
             
             if sum(to_fix)>0:    
                 if verbose: log.write(" -Filling CHR and POS columns using chr:pos:ref:alt format variants in rsID column...")
-                sumstats.loc[to_fix,chrom] = sumstats.loc[to_fix,rsid].str.split(':|_|-',n=2).str.get(0)
-                sumstats.loc[to_fix,pos] = sumstats.loc[to_fix,rsid].str.split(':|_|-',n=2).str.get(1)
+                sumstats.loc[to_fix,chrom] = sumstats.loc[to_fix,rsid].str.split(':|_|-',n=2).str[0]
+                sumstats.loc[to_fix,pos] = sumstats.loc[to_fix,rsid].str.split(':|_|-',n=2).str[1]
                 #sumstats.loc[to_fix,pos] = np.floor(pd.to_numeric(sumstats.loc[to_fix,rsid].str.split(':|_|-',x).get(1), errors='coerce')).astype('Int64')
                 #sumstats.loc[to_fix,status] = vchange_status(sumstats.loc[to_fix,status], 4, "98765432","00000000").astype("string")  
                 
@@ -234,7 +234,9 @@ def fixID(sumstats,
                 
                 #pattern = r"\w\w\w[0][01267][01234]\w"  
                 pattern = r"\w\w\w\w[0123][01267][01234]"
-                matched_index = sumstats[status].str.match(pattern) 
+                matched_index = sumstats[status].str.match(pattern)
+                if forcefixid is True:
+                    matched_index = to_fix
                 to_full_fix = matched_index & to_fix 
                 
                 
@@ -258,6 +260,7 @@ def fixID(sumstats,
             after_number=sum(sumstats[snpid].isna())
             if verbose: log.write(" -Fixed "+ str(pre_number - after_number) +" variants ID...")
         elif verbose: log.write(" -ID unfixable: no CHR and POS columns or no SNPID. ")
+    if verbose: log.write("Finished checking IDs successfully!")
     return sumstats
 
 ###############################################################################################################
@@ -267,28 +270,29 @@ def fixID(sumstats,
 #20220514 
 def removedup(sumstats,mode="dm",chrom="CHR",pos="POS",snpid="SNPID",ea="EA",nea="NEA",rsid="rsID",keep='first',keep_col="P",remove=False,keep_ascend=True,verbose=True,log=Log()):
     '''
-    remove duplicate SNPs based on  1. SNPID, EA, and NEA
-    remove duplicate SNPs based on  2. rsID for non-NA variants
-    remove multiallelic SNPs based on  3. chr pos 
+    remove duplicate SNPs based on 1. SNPID,
+    remove duplicate SNPs based on 2. CHR, POS, EA, and NEA
+    remove duplicate SNPs based on 3. rsID
+    remove multiallelic SNPs based on 4. CHR, POS
     '''
     
     if keep_col is not None : 
         if keep_col in sumstats.columns:
-            if verbose: log.write("Start to sort the sumstats using" + keep_col +"...")
+            if verbose: log.write("Start to sort the sumstats using " + keep_col +"...")
             sumstats = sumstats.sort_values(by=keep_col,ascending=keep_ascend)
         else:
             if verbose: log.write("Column" + keep_col +" was not detected... skipping... ")
     total_number = len(sumstats)  
-    if (snpid in sumstats.columns) and (nea in sumstats.columns) and (ea in sumstats.columns) and "d" in mode:
-        if verbose: log.write("Start to remove duplicated variants based on snpid,ea and nea...")
+    
+    if (snpid in sumstats.columns) and "d" in mode:
+        if verbose: log.write("Start to remove duplicated variants based on snpid...")
         if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns)) 
         if verbose: log.write(" -Which variant to keep: ",  keep )   
         pre_number =len(sumstats)   
         if snpid in sumstats.columns:
-            sumstats = sumstats.loc[sumstats[snpid].isna() | (~sumstats.duplicated(subset=[snpid,ea,nea], keep=keep)),:]
+            sumstats = sumstats.loc[sumstats[snpid].isna() | (~sumstats.duplicated(subset=[snpid], keep=keep)),:]
             after_number=len(sumstats)   
-            if verbose:  log.write(" -Removed ",pre_number -after_number ," based on SNPID, EA, and NEA...")
-     
+            if verbose:  log.write(" -Removed ",pre_number -after_number ," based on SNPID...")
     
     if (rsid in sumstats.columns) and ("d" in mode):
         pre_number =len(sumstats)
@@ -296,7 +300,17 @@ def removedup(sumstats,mode="dm",chrom="CHR",pos="POS",snpid="SNPID",ea="EA",nea
         sumstats = sumstats.loc[sumstats[rsid].isna() | (~sumstats.duplicated(subset=rsid, keep=keep)),:]
         after_number=len(sumstats)   
         if verbose:  log.write(" -Removed ",pre_number -after_number ," based on rsID...")
-      
+    
+    if (chrom in sumstats.columns) and (pos in sumstats.columns) and (nea in sumstats.columns) and (ea in sumstats.columns) and "d" in mode:
+        if verbose: log.write("Start to remove duplicated variants based on CHR,POS,EA and NEA...")
+        if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns)) 
+        if verbose: log.write(" -Which variant to keep: ",  keep )   
+        pre_number =len(sumstats)   
+        if snpid in sumstats.columns:
+            sumstats = sumstats.loc[(~sumstats[[chrom,pos,ea,nea]].all(axis=1)) | (~sumstats.duplicated(subset=[chrom,pos,ea,nea], keep=keep)),:]
+            after_number=len(sumstats)   
+            if verbose:  log.write(" -Removed ",pre_number -after_number ," based on CHR,POS,EA and NEA...") 
+        
     if (chrom in sumstats.columns) and (pos in sumstats.columns) and "m" in mode:
         pre_number =len(sumstats) 
         if verbose: log.write("Start to remove multiallelic variants based on chr:pos...")    
@@ -305,16 +319,18 @@ def removedup(sumstats,mode="dm",chrom="CHR",pos="POS",snpid="SNPID",ea="EA",nea
         after_number=len(sumstats)  
         if verbose:  log.write(" -Removed ",total_number -after_number," multiallelic variants in total.")   
     after_number=len(sumstats)   
+    
     if verbose:  log.write(" -Removed ",total_number -after_number," duplicates in total.")
     if keep_col is not None : 
         if verbose: log.write(" -Sort the coordinates...")
-        sumstats = sortcoordinate(sumstats)
+        sumstats = sortcoordinate(sumstats,verbose=False)
     if remove is True:
         if verbose: log.write(" -Removing NAs...")
         pre_number =len(sumstats) 
         sumstats = sumstats.loc[~sumstats.isna().any(axis=1),:]
         after_number=len(sumstats) 
         if verbose:  log.write(" -Removed ",pre_number -after_number," variants with NA values.")  
+    if verbose: log.write("Finished removing successfully!")
     return sumstats
 
 ###############################################################################################################
@@ -392,6 +408,7 @@ def fixchr(sumstats,chrom="CHR",status="STATUS",add_prefix="",x="X",y="Y",mt="MT
         sumstats.loc[:,chrom] = np.floor(pd.to_numeric(sumstats.loc[:,chrom], errors='coerce')).astype('Int64')
         #categories = [str(i) for i in range(26)]+[pd.NA]
         #sumstats.loc[:,chrom]= pd.Categorical(sumstats.loc[:,chrom],categories=categories,ordered=True)
+        if verbose: log.write("Finished fixing chromosome notation successfully!")
         return sumstats
     
     
@@ -431,7 +448,7 @@ def fixpos(sumstats,pos="POS",status="STATUS",remove=False, verbose=True,limit=2
             if verbose: log.write(" -Removed "+str(all_var_num - remain_var_num)+" variants with bad positions.")        
         
         if verbose: log.write(" -Converted all position to datatype Int64.")
-        if verbose: log.write(" -Fixed basepair position successfully.")
+        if verbose: log.write("Finished fixing basepair position successfully!")
         return sumstats
     
 ###############################################################################################################    
@@ -478,8 +495,7 @@ def fixallele(sumstats,ea="EA", nea="NEA",status="STATUS",remove=False,verbose=T
         sumstats.loc[is_eanea_fixed&is_normalized,status]      = vchange_status(sumstats.loc[is_eanea_fixed&is_normalized, status],  5,"4","3")
         remain_var_num = len(sumstats)
               
-        
-        if verbose: log.write(" -Fixed allele successfully.")
+        if verbose: log.write("Finished fixing allele successfully!")
         return sumstats
 
 ###############################################################################################################   
@@ -494,7 +510,8 @@ def parallelnormalizeallele(sumstats,pos="POS",nea="NEA",ea="EA" ,status="STATUS
     if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))   
     variants_to_check = sumstats[status].str.match(r'\w\w\w\w[45]\w\w', case=False, flags=0, na=False)
     if sum(variants_to_check)==0:
-        log.write(" -No available variants to normalize..")
+        if verbose: log.write(" -No available variants to normalize..")
+        if verbose: log.write("Finished normalizing variants successfully!")
         return sumstats
     ###############################################################################################################
     if sum(variants_to_check)>0:
@@ -532,6 +549,7 @@ def parallelnormalizeallele(sumstats,pos="POS",nea="NEA",ea="EA" ,status="STATUS
     #indel_to_snp = sumstats[status].str.match(r'\w\w\w\w[3]\w\w', case=False, flags=0, na=False) & (sumstats[ea].str.len()==1) &(sumstats[nea].str.len()==1)
     #if sum(indel_to_snp)>0:
     #    sumstats.loc[indel_to_snp,status] = vchange_status(sumstats.loc[indel_to_snp,status], 5,"5","3")
+    if verbose: log.write("Finished normalizing variants successfully!")
     return sumstats
 
 def normalizeallele(sumstats,pos="POS" ,nea="NEA",ea="EA",status="STATUS"):
@@ -587,7 +605,23 @@ def normalizevariant(pos,a,b,status):
 
 ###############################################################################################################
 #20220426
-def sanitycheckstats(sumstats,coltocheck=["P","MLOG10P","BETA","SE","EAF","CHISQ","N","OR","OR_95L","OR_95U","STATUS"],verbose=True,log=Log()):
+def sanitycheckstats(sumstats,
+                     coltocheck=["P","MLOG10P","Z","BETA","SE","EAF","CHISQ","N","OR","OR_95L","OR_95U","STATUS"],
+                     n=(0,float("Inf")),
+                     eaf=(0,1),
+                     mac=(5,float("Inf")),
+                     chisq=(0,float("Inf")),
+                     z=(-37.5,37.5),
+                     p=(5e-300,1),
+                     mlog10p=(0,float("Inf")),
+                     beta=(-10,10),
+                     se=(0,float("Inf")),
+                     OR=(-10,10),
+                     OR_95L=(0,float("Inf")),
+                     OR_95U=(0,float("Inf")),
+                     info=(0,float("Inf")),
+                     verbose=True,
+                     log=Log()):
     '''
     Sanity check:
         N:      Int32    , N>0 , 
@@ -606,105 +640,115 @@ def sanitycheckstats(sumstats,coltocheck=["P","MLOG10P","BETA","SE","EAF","CHISQ
     if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))   
     cols_to_check=[]
     oringinal_number=len(sumstats)
-    
+    sumstats = sumstats.copy()
     pre_number=len(sumstats)
     if "N" in coltocheck and "N" in sumstats.columns:
         cols_to_check.append("N")
-        if verbose: log.write(" -Checking if N is Int64 and N>0 ...") 
+        if verbose: log.write(" -Checking if ",n[0],"<=N<=",n[1]," ...") 
         sumstats.loc[:,"N"] = np.floor(pd.to_numeric(sumstats.loc[:,"N"], errors='coerce')).astype("Int32")
+        sumstats = sumstats.loc[(sumstats["N"]>=n[0]) & (sumstats["N"]<=n[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad N.") 
             
     pre_number=len(sumstats)    
     if "EAF" in coltocheck and "EAF" in sumstats.columns:
         cols_to_check.append("EAF")
-        if verbose: log.write(" -Checking if 0<= EAF <=1 ...") 
+        if verbose: log.write(" -Checking if ",eaf[0],"<=EAF<=",eaf[1]," ...") 
         sumstats.loc[:,"EAF"] = pd.to_numeric(sumstats.loc[:,"EAF"], errors='coerce').astype("float32")
-        sumstats = sumstats.loc[(sumstats["EAF"]>=0) & (sumstats["EAF"]<=1),:]
-        sumstats.loc[:,"EAF"] = sumstats.loc[:,"EAF"]
+        sumstats = sumstats.loc[(sumstats["EAF"]>=eaf[0]) & (sumstats["EAF"]<=eaf[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad EAF.") 
 
     pre_number=len(sumstats)    
     if "EAF" in coltocheck and "EAF" in sumstats.columns and "N" in coltocheck and "N" in sumstats.columns:
-        if verbose: log.write(" -Checking if MAC >=5 ...") 
-        mac5 = (sumstats.loc[:,"EAF"] * sumstats.loc[:,"N"] > 5) | ((1-sumstats.loc[:,"EAF"]) * sumstats.loc[:,"N"] > 5)
-        sumstats = sumstats.loc[mac5,:]
+        if verbose: log.write(" -Checking if ",mac[0],"<=MAC<=",mac[1]," ...") 
+        sumstats["_MAF"]=sumstats["EAF"]
+        sumstats.loc[sumstats["EAF"]>0.5,"_MAF"] = 1 - sumstats.loc[sumstats["EAF"]>0.5,"EAF"]
+        macl = (sumstats.loc[:,"_MAF"] * sumstats.loc[:,"N"] >= mac[0]) 
+        macu = (sumstats.loc[:,"_MAF"] * sumstats.loc[:,"N"] <= mac[1]) 
+        sumstats = sumstats.loc[macl&macu,:]
+        sumstats = sumstats.drop(labels="_MAF",axis=1)
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad MAC.") 
     
     pre_number=len(sumstats)    
     if "CHISQ" in coltocheck and "CHISQ" in sumstats.columns:
         cols_to_check.append("CHISQ")
-        if verbose: log.write(" -Checking if CHISQ>0 ...") 
+        if verbose: log.write(" -Checking if ",chisq[0],"<CHISQ<",chisq[1]," ...") 
         sumstats.loc[:,"CHISQ"] = pd.to_numeric(sumstats.loc[:,"CHISQ"], errors='coerce').astype("float32")
-        sumstats = sumstats.loc[(sumstats["CHISQ"]>=0),:]
+        sumstats = sumstats.loc[(sumstats["CHISQ"]>=chisq[0]) & (sumstats["CHISQ"]<=chisq[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad CHISQ.") 
-            
+    
+    pre_number=len(sumstats)    
+    if "Z" in coltocheck and "Z" in sumstats.columns:
+        cols_to_check.append("Z")
+        if verbose: log.write(" -Checking if ",z[0],"<Z<",z[1]," ...") 
+        sumstats.loc[:,"Z"] = pd.to_numeric(sumstats.loc[:,"Z"], errors='coerce').astype("float32")
+        sumstats = sumstats.loc[(sumstats["Z"]>=z[0]) & (sumstats["Z"]<=z[1]),:]
+        after_number=len(sumstats)
+        if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad Z.") 
+    
     pre_number=len(sumstats) 
     if "P" in coltocheck and "P" in sumstats.columns:
         cols_to_check.append("P")
-        if verbose: log.write(" -Checking if 0< P <5e-300 ...") 
+        if verbose: log.write(" -Checking if ",p[0],"<= P <=",p[1]," ...") 
         sumstats.loc[:,"P"] = pd.to_numeric(sumstats.loc[:,"P"], errors='coerce')
-        sumstats = sumstats.loc[sumstats["P"]>=0,:]
-        sumstats.loc[sumstats["P"]<5e-300,"P"] = 5e-300
+        sumstats = sumstats.loc[(sumstats["P"]>=p[0]) & (sumstats["P"]<=p[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad P.") 
     
     pre_number=len(sumstats)
     if "MLOG10P" in coltocheck and "MLOG10P" in sumstats.columns:
         cols_to_check.append("MLOG10P")
-        if verbose: log.write(" -Checking if MLOG10P>=0 ...") 
+        if verbose: log.write(" -Checking if ",mlog10p[0],"<MLOG10P<",mlog10p[1]," ...") 
         sumstats.loc[:,"MLOG10P"] = pd.to_numeric(sumstats.loc[:,"MLOG10P"], errors='coerce').astype("float32")
-        sumstats = sumstats.loc[(sumstats["MLOG10P"]>=0),:]
+        sumstats = sumstats.loc[(sumstats["MLOG10P"]>=mlog10p[0]) & (sumstats["MLOG10P"]<=mlog10p[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad MLOG10P.") 
     
     pre_number=len(sumstats)    
     if "BETA" in coltocheck and "BETA" in sumstats.columns:
         cols_to_check.append("BETA")
-        if verbose: log.write(" -Checking if abs(BETA)<10 ...") 
+        if verbose: log.write(" -Checking if ",beta[0],"<BETA)<",beta[1]," ...") 
         sumstats.loc[:,"BETA"] = pd.to_numeric(sumstats.loc[:,"BETA"], errors='coerce').astype("float32")
-        sumstats = sumstats.loc[np.abs(sumstats["BETA"])<10,:]
-        sumstats.loc[:,"BETA"] = sumstats.loc[:,"BETA"]
+        sumstats = sumstats.loc[(sumstats["BETA"]>=beta[0]) & (sumstats["BETA"]<=beta[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad BETA.") 
             
     pre_number=len(sumstats)    
     if "SE" in coltocheck and "SE" in sumstats.columns:
         cols_to_check.append("SE")
-        if verbose: log.write(" -Checking if SE >0 ...") 
+        if verbose: log.write(" -Checking if ",se[0],"<SE<",se[1]," ...") 
         sumstats.loc[:,"SE"] = pd.to_numeric(sumstats.loc[:,"SE"], errors='coerce').astype("float32")
-        sumstats = sumstats.loc[sumstats["SE"]>0,:]
-        sumstats.loc[:,"SE"] = sumstats.loc[:,"SE"]
+        sumstats = sumstats.loc[(sumstats["SE"]>=se[0]) & (sumstats["SE"]<=se[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad SE.") 
             
     pre_number=len(sumstats)    
     if "OR" in coltocheck and "OR" in sumstats.columns:
         cols_to_check.append("OR")
-        if verbose: log.write(" -Checking if -10<log(OR)<10 ...") 
+        if verbose: log.write(" -Checking if ",OR[0],"<log(OR)<",OR[1]," ...") 
         sumstats.loc[:,"OR"] = pd.to_numeric(sumstats.loc[:,"OR"], errors='coerce').astype("float32")
-        sumstats = sumstats.loc[(np.log(sumstats["OR"])>-10) & (np.log(sumstats["OR"])<10),:]
+        sumstats = sumstats.loc[(np.log(sumstats["OR"])>OR[0]) & (np.log(sumstats["OR"])<OR[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad OR.") 
             
     pre_number=len(sumstats)   
     if "OR_95L" in coltocheck and "OR_95L" in sumstats.columns:
         cols_to_check.append("OR_95L")
-        if verbose: log.write(" -Checking if OR_95L>0 ...") 
+        if verbose: log.write(" -Checking if ",OR_95L[0],"<OR_95L<",OR_95L[1]," ...") 
         sumstats.loc[:,"OR_95L"] = pd.to_numeric(sumstats.loc[:,"OR_95L"], errors='coerce').astype("float32")
-        sumstats = sumstats.loc[sumstats["OR_95L"]>0,:]
+        sumstats = sumstats.loc[(sumstats["OR_95L"]>OR_95L[0]) & (sumstats["OR_95L"]<OR_95L[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad OR_95L.") 
             
     pre_number=len(sumstats)    
     if "OR_95U" in coltocheck and "OR_95U" in sumstats.columns:
         cols_to_check.append("OR_95U")
-        if verbose: log.write(" -Checking if OR_95L>0 ...") 
+        if verbose: log.write(" -Checking if ",OR_95U[0],"<OR_95U<",OR_95U[1]," ...") 
         sumstats.loc[:,"OR_95U"] = pd.to_numeric(sumstats.loc[:,"OR_95U"], errors='coerce').astype("float32")
-        sumstats = sumstats.loc[sumstats["OR_95U"]>0,:]
+        sumstats = sumstats.loc[(sumstats["OR_95U"]>OR_95U[0]) & (["OR_95U"]<OR_95U[1]),:]
         after_number=len(sumstats)
         if verbose: log.write(" -Removed "+str(pre_number - after_number)+" variants with bad OR_95U.") 
     
@@ -713,7 +757,7 @@ def sanitycheckstats(sumstats,coltocheck=["P","MLOG10P","BETA","SE","EAF","CHISQ
         cols_to_check.append("STATUS")
         if verbose: log.write(" -Checking STATUS...") 
         categories = {str(j+i) for j in [1900000,3800000,9700000,9800000,9900000] for i in range(0,100000)}
-        sumstats["STATUS"] = pd.Categorical(sumstats["STATUS"],categories=categories)
+        sumstats.loc[:,"STATUS"] = pd.Categorical(sumstats["STATUS"],categories=categories)
         if verbose: log.write(" -Coverting STAUTUS to category.") 
     
     
@@ -721,6 +765,7 @@ def sanitycheckstats(sumstats,coltocheck=["P","MLOG10P","BETA","SE","EAF","CHISQ
     sumstats = sumstats.dropna(subset=cols_to_check)
     after_number=len(sumstats)
     if verbose: log.write(" -Removed "+str(oringinal_number - after_number)+" variants with bad statistics in total.") 
+    if verbose: log.write("Finished sanity check successfully!")
     return sumstats
 
 ###############################################################################################################
@@ -785,7 +830,7 @@ def flipallelestats(sumstats,status="STATUS",verbose=True,log=Log()):
             sumstats.loc[matched_index,"EAF"] =   1 - sumstats.loc[matched_index,"EAF"].values
         if "OR" in sumstats.columns:
             if verbose: log.write(" -Flipping column: OR = 1 / OR...") 
-            sumstats.loc[matched_index,"OR"] =   1 / sumstats.loc[matched_index,"freq"].values
+            sumstats.loc[matched_index,"OR"] =   1 / sumstats.loc[matched_index,"OR"].values
         if "OR_95L" in sumstats.columns:
             if verbose: log.write(" -Flipping column: OR_95L = 1 / OR_95L...") 
             sumstats.loc[matched_index,"OR_95L"] =   1 / sumstats.loc[matched_index,"OR_95L"].values
@@ -857,7 +902,7 @@ def flipallelestats(sumstats,status="STATUS",verbose=True,log=Log()):
         #change status    
         if verbose: log.write(" -Changed the status for flipped variants:  xxxxx[012]5: ->  xxxxx[012]2") 
         sumstats.loc[matched_index,status] = vchange_status(sumstats.loc[matched_index,status], 7, "5","2")
-
+    if verbose: log.write("Finished converting successfully!")
     return sumstats
 ###############################################################################################################
 
@@ -937,7 +982,7 @@ def parallelizeliftovervariant(sumstats,n_cores=1,chrom="CHR", pos="POS", from_b
     sumstats = fixchr(sumstats,chrom=chrom,add_prefix="",remove=remove, verbose=False)
     sumstats = fixpos(sumstats,pos=pos,remove=remove, verbose=False)
     
-    if verbose: log.write(" -Liftover is performed successfully!")
+    if verbose: log.write("Finished liftover successfully!")
     return sumstats
 
 ###############################################################################################################
@@ -986,17 +1031,16 @@ def sortcoordinate(sumstats,chrom="CHR",pos="POS",reindex=True,verbose=True,log=
     
     #sumstats[chrom] = sumstats[chrom].map(chromosome_number_to_string)
     #sumstats[chrom] = sumstats[chrom].astype("string")
-    
+    if verbose: log.write("Finished sorting genome coordinates successfully!")
     return sumstats
 ###############################################################################################################
 #20220426
-def sortcolumn(sumstats,verbose=True,log=Log()):
-    if verbose: log.write("Start to reorder the columns...")
-    if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))   
-    order = [
+def sortcolumn(sumstats,verbose=True,log=Log(),order = [
         "SNPID","rsID", "CHR", "POS", "EA", "NEA", "EAF", "BETA", "SE", "Z",
         "CHISQ", "P", "MLOG10P", "OR", "OR_SE", "OR_95L", "OR_95U", "INFO", "N","DIRECTION","STATUS"
-           ]
+           ]):
+    if verbose: log.write("Start to reorder the columns...")
+    if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))   
     
     output_columns = []
     for i in order:
@@ -1005,6 +1049,7 @@ def sortcolumn(sumstats,verbose=True,log=Log()):
         if i not in order: output_columns.append(i)
     if verbose: log.write(" -Reordering columns to    :", ",".join(output_columns))
     sumstats = sumstats.loc[:, output_columns]
+    if verbose: log.write("Finished sorting columns successfully!")
     return sumstats
 
 ################################################################################################################
@@ -1034,4 +1079,4 @@ def check_col(df,*args):
     return True
             
     
-    
\ No newline at end of file
+    
